@@ -206,6 +206,58 @@ bench_setup_rectangular_wgs84_2nd_to_1st_quadrant_long(gpointer user_data)
   data->geo_distance_proc = GET(data->context, "geo_distance");
 }
 
+static void
+bench_setup_rectangular_wgs84_3rd_to_4th_quadrant_short(gpointer user_data)
+{
+  BenchmarkData *data = user_data;
+
+  bench_setup_common(user_data);
+  bench_setup_points(user_data,
+                     "-56880000x-172310000",
+                     "-122100000x66300000",
+                     GRN_DB_WGS84_GEO_POINT);
+  data->geo_distance_proc = GET(data->context, "geo_distance");
+}
+
+static void
+bench_setup_rectangular_wgs84_4th_to_3rd_quadrant_short(gpointer user_data)
+{
+  BenchmarkData *data = user_data;
+
+  bench_setup_common(user_data);
+  bench_setup_points(user_data,
+                     "-122100000x66300000",
+                     "-56880000x-172310000",
+                     GRN_DB_WGS84_GEO_POINT);
+  data->geo_distance_proc = GET(data->context, "geo_distance");
+}
+
+static void
+bench_setup_rectangular_wgs84_3rd_to_4th_quadrant_long(gpointer user_data)
+{
+  BenchmarkData *data = user_data;
+
+  bench_setup_common(user_data);
+  bench_setup_points(user_data,
+                     "-56880000x-172310000",
+                     "-121926000x544351000",
+                     GRN_DB_WGS84_GEO_POINT);
+  data->geo_distance_proc = GET(data->context, "geo_distance");
+}
+
+static void
+bench_setup_rectangular_wgs84_4th_to_3rd_quadrant_long(gpointer user_data)
+{
+  BenchmarkData *data = user_data;
+
+  bench_setup_common(user_data);
+  bench_setup_points(user_data,
+                     "-121926000x544351000",
+                     "-56880000x-172310000",
+                     GRN_DB_WGS84_GEO_POINT);
+  data->geo_distance_proc = GET(data->context, "geo_distance");
+}
+
 static void
 bench_setup_spherical_wgs84(gpointer user_data)
 {
@@ -302,6 +354,14 @@ main(int argc, gchar **argv)
             rectangular_wgs84_1st_to_2nd_quadrant_long);
   REGISTER("rectangular (WGS84 San Francisco to Tokyo)",
             rectangular_wgs84_2nd_to_1st_quadrant_long);
+  REGISTER("rectangular (WGS84 Brasplia to Cape Town)",
+            rectangular_wgs84_3rd_to_4th_quadrant_short);
+  REGISTER("rectangular (WGS84 Cape Town to Brasplia)",
+            rectangular_wgs84_4th_to_3rd_quadrant_short);
+  REGISTER("rectangular (WGS84 Brasplia to Sydney)",
+            rectangular_wgs84_3rd_to_4th_quadrant_long);
+  REGISTER("rectangular (WGS84 Sydney to Brasplia)",
+            rectangular_wgs84_4th_to_3rd_quadrant_long);
   REGISTER("spherical (WGS84)", spherical_wgs84);
   REGISTER("spherical (TOKYO)", spherical_tgs);
   REGISTER("hubeny (WGS84)", hubeny_wgs84);
